@@ -4,13 +4,14 @@
     SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 */
 
+#include <algorithm>
+
 #include <QCoreApplication>
 #include <QCommandLineParser>
 #include <QCommandLineOption>
 #include <QDateTime>
 #include <QFile>
 #include <QTextStream>
-#include <QStandardPaths>
 
 #include <KAboutData>
 #include <KLocalizedString>
@@ -39,13 +40,24 @@ QString colorString(const QString& input, int color)
     }
 }
 
+inline KFileMetaData::PropertyMultiMap variantToPropertyMultiMap(const QVariantMap &varMap)
+{
+    KFileMetaData::PropertyMultiMap propMap;
+    QVariantMap::const_iterator it = varMap.constBegin();
+    for (; it != varMap.constEnd(); ++it) {
+        int p = it.key().toInt();
+        propMap.insert(static_cast<KFileMetaData::Property::Property>(p), it.value());
+    }
+    return propMap;
+}
+
 int main(int argc, char* argv[])
 {
     QCoreApplication app(argc, argv);
 
     KAboutData aboutData(QStringLiteral("balooshow"),
                          i18n("Baloo Show"),
-                         PROJECT_VERSION,
+                         QStringLiteral(PROJECT_VERSION),
                          i18n("The Baloo data Viewer - A debugging tool"),
                          KAboutLicense::GPL,
                          i18n("(c) 2012, Vishesh Handa"));
@@ -77,7 +89,7 @@ int main(int argc, char* argv[])
         bool ok;
         devId = parser.value(QStringLiteral("d")).toULong(&ok, 10);
         if (!ok) {
-            devId = parser.value(QStringLiteral("d")).toULong(&ok, 0);
+            devId = parser.value(QStringLiteral("d")).toULong(&ok, 16);
         }
     }
 
@@ -119,22 +131,24 @@ int main(int argc, char* argv[])
                 fid = fsFid;
             } else {
                 bool ok;
-                fid = url.toULongLong(&ok);
+                fid = url.toULongLong(&ok, 10);
+                if (!ok) {
+                    fid = url.toULongLong(&ok, 16);
+                }
                 if (!ok) {
                     stream << i18n("%1: Not a valid url or document id", url) << '\n';
                     continue;
                 }
-                if (!tr.hasDocument(fid)) {
-                    stream << i18n("%1: No index information found", url) << '\n';
-                    continue;
-                }
                 url = QFile::decodeName(tr.documentUrl(fid));
                 internalUrl = url;
             }
 
         } else {
             bool ok;
-            quint32 inode = url.toULong(&ok);
+            quint32 inode = url.toULong(&ok, 10);
+            if (!ok) {
+                inode = url.toULong(&ok, 16);
+            }
             if (!ok) {
                 stream << i18n("%1: Failed to parse inode number", url) << '\n';
                 continue;
@@ -145,19 +159,17 @@ int main(int argc, char* argv[])
             internalUrl = url;
         }
 
-        bool hasFile = tr.hasDocument(fid);
-        if (hasFile) {
-            stream << colorString(QString::number(fid), 31);
-            stream << QLatin1String(" ");
-            stream << colorString(QString::number(Baloo::idToDeviceId(fid)), 28);
-            stream << QLatin1String(" ");
-            stream << colorString(QString::number(Baloo::idToInode(fid)), 28);
-            stream << QLatin1String(" ");
+        if (fid) {
+            stream << colorString(QString::number(fid, 16), 31) << ' ';
+            stream << colorString(QString::number(Baloo::idToDeviceId(fid)), 28) << ' ';
+            stream << colorString(QString::number(Baloo::idToInode(fid)), 28) << ' ';
+        }
+        if (fid && tr.hasDocument(fid)) {
             stream << colorString(url, 32);
             if (!internalUrl.isEmpty() && internalUrl != url) {
                 // The document is know by a different name inside the DB,
                 // e.g. a hardlink, or untracked rename
-                stream << QLatin1String(" [") << internalUrl << QChar(']');
+                stream << QLatin1String(" [") << internalUrl << QLatin1Char(']');
             }
             stream << '\n';
         }
@@ -167,22 +179,21 @@ int main(int argc, char* argv[])
         }
 
         Baloo::DocumentTimeDB::TimeInfo time = tr.documentTimeInfo(fid);
-        stream << QLatin1String("\tMtime: ") << time.mTime << QChar(' ')
+        stream << QStringLiteral("\tMtime: %1 ").arg(time.mTime)
                << QDateTime::fromSecsSinceEpoch(time.mTime).toString(Qt::ISODate)
-               << QLatin1String("\n\tCtime: ") << time.cTime << QChar(' ')
+               << QStringLiteral("\n\tCtime: %1 ").arg(time.cTime)
                << QDateTime::fromSecsSinceEpoch(time.cTime).toString(Qt::ISODate)
               << '\n';
 
         const QJsonDocument jdoc = QJsonDocument::fromJson(tr.documentData(fid));
         const QVariantMap varMap = jdoc.object().toVariantMap();
-        KFileMetaData::PropertyMap propMap = KFileMetaData::toPropertyMap(varMap);
-        KFileMetaData::PropertyMap::const_iterator it = propMap.constBegin();
+        KFileMetaData::PropertyMultiMap propMap = variantToPropertyMultiMap(varMap);
         if (!propMap.isEmpty()) {
             stream << "\tCached properties:" << '\n';
         }
-        for (; it != propMap.constEnd(); ++it) {
+        for (auto it = propMap.constBegin(); it != propMap.constEnd(); ++it) {
             QString str;
-            if (it.value().type() == QVariant::List) {
+            if (it.value().typeId() == QMetaType::QVariantList) {
                 QStringList list;
                 const auto vars = it.value().toList();
                 for (const QVariant& var : vars) {
@@ -208,56 +219,61 @@ int main(int argc, char* argv[])
                     ba.append(arr);
                     ba.append(' ');
                 }
-                return QString(ba);
+                return QString::fromUtf8(ba);
             };
 
+            auto propertiesBegin = std::stable_partition(terms.begin(), terms.end(),
+                [](const auto & t) { return t.isEmpty() || t[0] < 'A' || t[0] > 'Z'; });
+            const QVector<QByteArray> propertyTerms{propertiesBegin, terms.end()};
+            terms.erase(propertiesBegin, terms.end());
+
             stream << "\n" << i18n("Internal Info") << "\n";
-            stream << i18n("Terms: %1", join(terms)) << "\n";
             stream << i18n("File Name Terms: %1", join(fileNameTerms)) << "\n";
             stream << i18n("%1 Terms: %2", QStringLiteral("XAttr"), join(xAttrTerms)) << '\n';
+            stream << i18n("Plain Text Terms: %1", join(terms)) << "\n";
+            stream << i18n("Property Terms: %1", join(propertyTerms)) << "\n";
 
             QHash<int, QStringList> propertyWords;
             KLocalizedString errorPrefix = ki18nc("Prefix string for internal errors", "Internal Error - %1");
 
-            for (const QByteArray& arr : terms) {
+            for (const QByteArray& arr : propertyTerms) {
                 auto arrAsPrintable = [arr]() {
                     return QString::fromLatin1(arr.toPercentEncoding());
                 };
 
                 if (arr.length() < 1) {
-                    auto error = QString("malformed term (short): '%1'\n").arg(arrAsPrintable());
+                    auto error = QStringLiteral("malformed term (short): '%1'\n").arg(arrAsPrintable());
                     stream << errorPrefix.subs(error).toString();
                     continue;
                 }
-                QString word = QString::fromUtf8(arr);
 
-                if (word[0].isUpper()) {
-                    if (word[0] == QLatin1Char('X')) {
+                const QString word = QString::fromUtf8(arr);
+
+                if (word[0] == QLatin1Char('X')) {
                         if (word.length() < 4) {
                             // 'X<num>-<value>
-                            auto error = QString("malformed property term (short): '%1' in '%2'\n").arg(word).arg(arrAsPrintable());
+                            auto error = QStringLiteral("malformed property term (short): '%1' in '%2'\n").arg(word, arrAsPrintable());
                             stream << errorPrefix.subs(error).toString();
                             continue;
                         }
-                        int posOfNonNumeric = word.indexOf('-', 2);
+                        const int posOfNonNumeric = word.indexOf(QLatin1Char('-'), 2);
                         if ((posOfNonNumeric < 0) || ((posOfNonNumeric + 1) == word.length())) {
-                            auto error = QString("malformed property term (no data): '%1' in '%2'\n").arg(word).arg(arrAsPrintable());
+                            auto error = QStringLiteral("malformed property term (no data): '%1' in '%2'\n").arg(word, arrAsPrintable());
                             stream << errorPrefix.subs(error).toString();
                             continue;
                         }
 
                         bool ok;
-                        QStringRef prop = word.midRef(1, posOfNonNumeric-1);
+                        const QStringView prop = QStringView(word).mid(1, posOfNonNumeric - 1);
                         int propNum = prop.toInt(&ok);
-                        QString value = word.mid(posOfNonNumeric + 1);
                         if (!ok) {
-                            auto error = QString("malformed property term (bad index): '%1' in '%2'\n").arg(prop).arg(arrAsPrintable());
+                            auto error = QStringLiteral("malformed property term (bad index): '%1' in '%2'\n").arg(prop, arrAsPrintable());
                             stream << errorPrefix.subs(error).toString();
                             continue;
                         }
 
+                        const QString value = word.mid(posOfNonNumeric + 1);
                         propertyWords[propNum].append(value);
-                    }
                 }
             }
 
